﻿using System.ComponentModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Input;

namespace BehaviorCollection.Controls
{

    /// <summary>
    /// Test Area
    /// </summary>
    [TemplatePart(Name = ResetItButtonStr, Type = typeof(FrameworkElement))]
    [TemplatePart(Name = DoItButtonStr, Type = typeof(FrameworkElement))]

    [TemplateVisualState(GroupName = CommonStatesStr, Name = NotPlayedStateStr)]
    [TemplateVisualState(GroupName = CommonStatesStr, Name = PlayedStateStr)]
    [TemplateVisualState(GroupName = WorkModeStatesStr, Name = AutoPlayStr)]
    [TemplateVisualState(GroupName = WorkModeStatesStr, Name = ButtonPlayStr)]

    public class TestArea : ContentControl, INotifyPropertyChanged
    {

        #region states constants
        private const string CommonStatesStr = "CommonStates";
        private const string WorkModeStatesStr = "WorkModeStates";
        private const string NotPlayedStateStr = "NotPlayedState";
        private const string PlayedStateStr = "PlayedState";
        private const string AutoPlayStr = "AutoPlay";
        private const string ButtonPlayStr = "ButtonPlay";
        #endregion

        #region template parts constants
        private const string ResetItButtonStr = "ResetItButton";
        private const string DoItButtonStr = "DoItButton";
        #endregion

        #region template parts fields
        private FrameworkElement doItButton;
        private FrameworkElement resetItButton;
        #endregion

        #region private fields
        private bool hasBeenPlayed;
        #endregion

        #region Has been played indicator
        /// <summary>
        /// Gets or sets a value indicating whether this instance has been played.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance has been played; otherwise, <c>false</c>.
        /// </value>
        public bool HasBeenPlayed
        {
            get { return hasBeenPlayed; }
            private set
            {
                if (value == hasBeenPlayed) return;
                hasBeenPlayed = value;
                VisualStateManager.GoToState(this, hasBeenPlayed ? PlayedStateStr : NotPlayedStateStr, true);
                doPropertyChanged("HasBeenPlayed");
            }
        }
        #endregion

        #region RunItCommand Property

        /// <summary>
        /// Gets or sets the run it command.
        /// </summary>
        /// <value>The run it command.</value>
        public ICommand RunItCommand
        {
            get { return (ICommand)GetValue(RunItCommandProperty); }
            set { SetValue(RunItCommandProperty, value); }
        }

        /// <summary>
        /// Gets or sets the run it command.
        /// </summary>
        public static readonly DependencyProperty RunItCommandProperty =
            DependencyProperty.Register(
                "RunItCommand", typeof(ICommand), typeof(TestArea),
                new PropertyMetadata(null, onRunItCommandChanged));

        /// <summary>
        /// Called when [run it command changed].
        /// </summary>
        /// <param name="d">The d.</param>
        /// <param name="e">The <see cref="System.Windows.DependencyPropertyChangedEventArgs"/> instance containing the event data.</param>
        private static void onRunItCommandChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            var ta = d as TestArea;
            if (ta != null) ta.HasBeenPlayed = false;
        }

        #endregion

        #region ResetItCommand Property

        /// <summary>
        /// Gets or sets the reset it command.
        /// </summary>
        /// <value>The reset it command.</value>
        public ICommand ResetItCommand
        {
            get { return (ICommand)GetValue(ResetItCommandProperty); }
            set { SetValue(ResetItCommandProperty, value); }
        }

        /// <summary>
        /// Gets or sets the reset it command.
        /// </summary>
        public static readonly DependencyProperty ResetItCommandProperty =
            DependencyProperty.Register(
                "ResetItCommand", typeof(ICommand), typeof(TestArea), null);

        #endregion

        #region Title Property

        /// <summary>
        /// Gets or sets the title.
        /// </summary>
        /// <value>The title.</value>
        public string Title
        {
            get { return (string)GetValue(TitleProperty); }
            set { SetValue(TitleProperty, value); }
        }

        /// <summary>
        /// Gets or sets the title.
        /// </summary>
        public static readonly DependencyProperty TitleProperty =
            DependencyProperty.Register(
                "Title", typeof(string), typeof(TestArea),
                new PropertyMetadata("Test", onTitleChanged));

        private static void onTitleChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            var testArea = d as TestArea;
            if (testArea != null) testArea.doPropertyChanged("Title");
        }

        #endregion

        #region public Run it method (to launch test even when template has no button)
        /// <summary>
        /// Runs the test.
        /// </summary>
        public void RunIt()
        {
            if (RunItCommand == null || !RunItCommand.CanExecute(null)) return;
            RunItCommand.Execute(null);
            VisualStateManager.GoToState(this, PlayedStateStr, true);
            HasBeenPlayed = true;
        }
        #endregion

        #region public Reset it method
        /// <summary>
        /// Resets it.
        /// </summary>
        public void ResetIt()
        {
            if (ResetItCommand != null && ResetItCommand.CanExecute(null))
            {
                ResetItCommand.Execute(null);
            }
            HasBeenPlayed = false;
        }
        #endregion

        #region ButtonPlay Property 

        /// <summary>
        /// Gets or sets a value indicating whether play mode is driven by buttons (do it/reset) or not.
        /// </summary>
        /// <value><c>true</c> if [button play]; otherwise, <c>false</c>.</value>
        public bool ButtonPlay
        {
            get { return (bool) GetValue(ButtonPlayProperty); }
            set { SetValue(ButtonPlayProperty, value); }
        }

        public static readonly DependencyProperty ButtonPlayProperty =
            DependencyProperty.Register(
                "ButtonPlay", typeof (bool), typeof (TestArea),
                new PropertyMetadata(false, OnButtonPlayChanged));

        private static void OnButtonPlayChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            var ta = d as TestArea;
            VisualStateManager.GoToState(ta, ta.ButtonPlay ? ButtonPlayStr : AutoPlayStr,true);
        }

        #endregion


        #region Ctor
        /// <summary>
        /// Initializes a new instance of the <see cref="TestArea"/> class.
        /// </summary>
        public TestArea()
        {
            DefaultStyleKey = typeof(TestArea);
            hasBeenPlayed = false;
            Loaded += TestArea_Loaded;
        }

        private void TestArea_Loaded(object sender, System.Windows.RoutedEventArgs e)
        {
            VisualStateManager.GoToState(this, NotPlayedStateStr, false);
            VisualStateManager.GoToState(this, ButtonPlay ? ButtonPlayStr : AutoPlayStr, true);
        }
        #endregion

        #region internal stuff
        /// <summary>
        /// When overridden in a derived class, is invoked whenever application code or internal processes (such as a rebuilding layout pass) call <see cref="M:System.Windows.Controls.Control.ApplyTemplate"/>. In simplest terms, this means the method is called just before a UI element displays in an application. For more information, see Remarks.
        /// </summary>
        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();
            doItButton = GetTemplateChild(DoItButtonStr) as FrameworkElement;
            resetItButton = GetTemplateChild(ResetItButtonStr) as FrameworkElement;
            if (resetItButton is ButtonBase)
                (resetItButton as ButtonBase).Click += resetItButton_Click;
            else if (resetItButton != null) resetItButton.MouseLeftButtonDown += resetItButton_MouseLeftButtonDown;
            if (doItButton is ButtonBase)
                (doItButton as ButtonBase).Click += doItButton_Click;
            else if (doItButton != null) doItButton.MouseLeftButtonDown += doItButton_MouseLeftButtonDown;
        }

        /// <summary>
        /// Handles the MouseLeftButtonDown event of the <see cref="resetItButton"/> control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Input.MouseButtonEventArgs"/> instance containing the event data.</param>
        void resetItButton_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            ResetIt();
        }

        /// <summary>
        /// Handles the Click event of the <see cref="resetItButton"/> control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.RoutedEventArgs"/> instance containing the event data.</param>
        void resetItButton_Click(object sender, RoutedEventArgs e)
        {
            ResetIt();
        }

        /// <summary>
        /// Handles the MouseLeftButtonDown event of the <see cref="doItButton"/> control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The 
        /// <see cref="System.Windows.Input.MouseButtonEventArgs"/> instance
        /// containing the event data.</param>
        void doItButton_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            RunIt();
        }

        /// <summary>
        /// Handles the Click event of the <see cref="doItButton"/> control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.RoutedEventArgs"/> instance containing the event data.</param>
        void doItButton_Click(object sender, RoutedEventArgs e)
        {
            RunIt();
        }
        #endregion

        #region PropetyChanged
        /// <summary>
        /// Occurs when a property value changes.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        private void doPropertyChanged(string propName)
        {
            var p = PropertyChanged;
            if (p == null) return;
            p(this, new PropertyChangedEventArgs(propName));
        }


        #endregion
    }
}
