﻿using System;
using System.ComponentModel;
using System.Windows;
using System.Windows.Data;
using System.Windows.Interactivity;
using System.Windows.Media.Animation;
using Od.Behaviors.Utilities;
using Od.Behaviors.Utilities.Expression.Samples.Interactivity.DataHelpers;

namespace Od.Behaviors.Animation
{
    /// <summary>
    /// Animate Visibility
    /// Debug + cleanup : OD 2010
    /// </summary>
    public class AnimateVisibilityBehavior : Behavior<FrameworkElement>
    {
        /// <summary>
        /// Animation executed when Visibility switch to Visible"
        /// </summary>
        [Description("Animation executed when Visibility switch to Visible")]
        public Storyboard VisibleAnimation { get; set; }

        /// <summary>
        /// Animation executed when Visibility switch to Collapsed"
        /// </summary>
        [Description("Animation executed when Visibility switch to Collapsed")]
        public Storyboard CollapsedAnimation { get; set; }


        bool useDefaultAnimation
        {
            get
            {
                return VisibleAnimation == null && CollapsedAnimation == null;
            }
        }

        readonly BindingListener listener;
        bool listenChanges;
        bool firstTime;

        Visibility visibilityToSet;
        double lastHeight;

        /// <summary>
        /// Initializes a new instance of the <see cref="AnimateVisibilityBehavior"/> class.
        /// </summary>
        public AnimateVisibilityBehavior()
        {
            listener = new BindingListener(visibilityChanged)
                           {
                               Binding = new Binding("Visibility") {Mode = BindingMode.TwoWay}
                           };

            listenChanges = true;
        }

        private void visibilityChanged(object sender, EventArgs e)
        {
            if (firstTime)
            {
                firstTime = false;
                return;
            }

            if (!listenChanges) return;

            listenChanges = false;
            visibilityToSet = AssociatedObject.Visibility;
            if (AssociatedObject.Visibility == Visibility.Collapsed) AssociatedObject.Visibility = Visibility.Visible;
            animateVisibility(visibilityToSet);
        }

        void animateVisibility(Visibility value)
        {
            if (useDefaultAnimation)
                defaultAnimation(value);
            else
                customAnimation(value);

        }

        void defaultAnimation(Visibility value)
        {
            Storyboard animation;

            if (value == Visibility.Visible)
                animation = TransitionHelper.BuildVisibilityAnimation
                    (AssociatedObject,
                     1,
                     lastHeight,
                     TimeSpan.FromMilliseconds(180),
                     TimeSpan.FromMilliseconds(160),
                     true);
            else
            {
                lastHeight = AssociatedObject.ActualHeight;
                AssociatedObject.Height = AssociatedObject.ActualHeight;
                animation = TransitionHelper.BuildVisibilityAnimation
                    (AssociatedObject,
                     0,
                     0,
                     TimeSpan.FromMilliseconds(180),
                     TimeSpan.FromMilliseconds(160),
                     false);
            }

            animation.Completed += visibilityAnimation_Completed;
            animation.Begin();
        }

        void customAnimation(Visibility value)
        {
            var animation = value == Visibility.Visible ? VisibleAnimation : CollapsedAnimation;

            if (animation != null)
            {
                animation.Completed -= visibilityAnimation_Completed;
                animation.Completed += visibilityAnimation_Completed;
                animation.Begin();
            }
            else
            {
                AssociatedObject.SetValue(UIElement.VisibilityProperty, visibilityToSet);
                listenChanges = true;
            }
        }

        private void visibilityAnimation_Completed(object sender, EventArgs e)
        {
            AssociatedObject.SetValue(UIElement.VisibilityProperty, visibilityToSet);
            listenChanges = true;
        }

        /// <summary>
        /// Attaching element
        /// </summary>
        protected override void OnAttached()
        {
            base.OnAttached();

            if (AssociatedObject.Visibility == Visibility.Collapsed)
            {
                AssociatedObject.Visibility = Visibility.Visible;
                lastHeight = AssociatedObject.ActualHeight;
                AssociatedObject.Visibility = Visibility.Collapsed;
            }

            firstTime = true;
            listener.Binding.Source = AssociatedObject;
            listener.Element = AssociatedObject;
        }

        /// <summary>
        /// Called when the behavior is being detached from its AssociatedObject, but before it has actually occurred.
        /// </summary>
        /// <remarks>Override this to unhook functionality from the AssociatedObject.</remarks>
        protected override void OnDetaching()
        {
            base.OnDetaching();

            listener.Binding.Source = null;
            listener.Element = null;
        }

    }
}
