﻿using System;
using System.Windows;
using System.Windows.Input;
using System.Windows.Interactivity;
using System.Windows.Media;
using System.Windows.Media.Animation;

namespace Od.Behaviors.Animation
{
    /// <summary>
    /// Bouncing plane
    /// Debug + Cleanup : OD 2010
    /// </summary>
    public class BouncingPlaneBehavior : Behavior<FrameworkElement>
    {

        /// <summary>
        /// Builds the key frames.
        /// </summary>
        /// <param name="mass">The mass.</param>
        /// <param name="springConstant">The spring constant.</param>
        /// <param name="dampening">The dampening.</param>
        /// <param name="forceMultiplier">The force multiplier.</param>
        /// <param name="directionalScalar">The directional scalar.</param>
        /// <param name="offset">The offset.</param>
        /// <returns></returns>
        private static DoubleAnimationUsingKeyFrames buildKeyFrames(double mass, double springConstant, double dampening, double forceMultiplier, double directionalScalar, double offset)
        {
            // In milliseconds
            const int sample = 75;
            const int duration = 10000;

            // Declare physics constants
            var lamda = dampening / (2 * Math.Sqrt(springConstant * mass));
            var omegaNot = Math.Sqrt(springConstant / mass);
            var omegaD = omegaNot * Math.Sqrt(1.0 - Math.Pow(lamda, 2.0));
            var amplitude = directionalScalar * forceMultiplier;

            var animation = new DoubleAnimationUsingKeyFrames();
            for (var i = 0; i < (duration / sample); i++)
            {
                var frame = new LinearDoubleKeyFrame
                                {
                                    KeyTime = KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(i * sample)),
                                    Value =
                                        offset +
                                        amplitude * Math.Exp(-1 * lamda * omegaNot * sample * i / 1000) *
                                        Math.Sin(omegaD * i * sample / 1000)
                                };
                animation.KeyFrames.Add(frame);
            }

            return animation;
        }

        /// <summary>
        /// Called after the behavior is attached to an AssociatedObject.
        /// </summary>
        /// <remarks>Override this to hook up functionality to the AssociatedObject.</remarks>
        protected override void OnAttached()
        {
            AssociatedObject.Projection = new PlaneProjection
                                              {
                                                  CenterOfRotationX = 0.5,
                                                  CenterOfRotationY = 0.5
                                              };
            AssociatedObject.MouseLeftButtonDown += onMouseLeftButtonDown;
            base.OnAttached();
        }

        /// <summary>
        /// Called when the behavior is being detached from its AssociatedObject, but before it has actually occurred.
        /// </summary>
        /// <remarks>Override this to unhook functionality from the AssociatedObject.</remarks>
        protected override void OnDetaching()
        {
            AssociatedObject.MouseLeftButtonDown -= onMouseLeftButtonDown;
            base.OnDetaching();
        }

        private void onMouseLeftButtonDown(object sender, MouseButtonEventArgs args)
        {
            if (args.OriginalSource == null)
            {
                return;
            }

            // Get click position relative to the clicked object
            var p = args.GetPosition((UIElement)args.OriginalSource);

            // Determine simple moment arm and direction against clicked object on both x & y axes.
            var directionalScalarY = (p.Y - (AssociatedObject.RenderSize.Height / 2)) / (AssociatedObject.RenderSize.Height / 2);
            var directionalScalarX = ((AssociatedObject.RenderSize.Width / 2) - p.X) / (AssociatedObject.RenderSize.Width / 2);

            var sb = new Storyboard();

            var da = buildKeyFrames(1.0, 80.0, 1.0, 35.0, directionalScalarX, 5.0);
            Storyboard.SetTarget(da, AssociatedObject.Projection);
            Storyboard.SetTargetProperty(da, new PropertyPath("RotationY"));
            sb.Children.Add(da);

            da = buildKeyFrames(1.0, 80.0, 1.0, 35.0, directionalScalarY, 5.0);
            Storyboard.SetTarget(da, AssociatedObject.Projection);
            Storyboard.SetTargetProperty(da, new PropertyPath("RotationX"));
            sb.Children.Add(da);

            sb.Begin();
        }
    }
}
