﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Interactivity;
using System.ComponentModel;

namespace Od.Behaviors.Animation
{
    /// <summary>
    /// Random children animation
    /// Debug and Cleanup : OD 2010
    /// </summary>
    public class RandomChildrenAnimationBehavior : Behavior<Canvas>
    {
        private double i;
        private Canvas hostPanel;

        static readonly Random ranNum = new Random();
        static Point parentWindowSize;

        bool arePropertiesSet;

        #region Property to Expose
        /// <summary>
        /// Gets or sets the speed.
        /// </summary>
        /// <value>The speed.</value>
        [Category("Animation Properties")]
        public double Speed
        {
            get { return (double)GetValue(SpeedProperty); }
            set { SetValue(SpeedProperty, value); }
        }

        /// <summary>
        /// Gets or sets the speed.
        /// </summary>
        public static readonly DependencyProperty SpeedProperty = DependencyProperty.Register("Speed", typeof(double), typeof(RandomChildrenAnimationBehavior), new PropertyMetadata(5.0));

        #endregion

        /// <summary>
        /// Called after the behavior is attached to an AssociatedObject.
        /// </summary>
        /// <remarks>Override this to hook up functionality to the AssociatedObject.</remarks>
        protected override void OnAttached()
        {
            setProperties();

            if (!DesignerProperties.GetIsInDesignMode(this))
            {
                CompositionTarget.Rendering += compositionTarget_Rendering;
            }
        }

        private void setProperties()
        {
            hostPanel = AssociatedObject;
            parentWindowSize = new Point(hostPanel.ActualWidth, hostPanel.ActualHeight);

            i = 0;

            foreach (FrameworkElement element in hostPanel.Children)
            {
                var speed = ranNum.NextDouble() * Speed;
                double radius = ranNum.Next(10, 20);

                var randomPosition = new Point(ranNum.Next((int)-element.ActualWidth,
                                               (int)(parentWindowSize.X + element.ActualWidth)), ranNum.Next((int)-element.ActualHeight,
                                               (int)(parentWindowSize.Y + element.ActualHeight)));

                if (element.Tag == null)
                {
                    element.Tag = new ElementProperties
                                      {
                                          Radius = radius,
                                          Speed = speed,
                                          CurrentPosition = new Point(Canvas.GetLeft(element), Canvas.GetTop(element)),
                                          RandomPosition = randomPosition
                                      };
                }
            }
        }

       private void compositionTarget_Rendering(object sender, EventArgs e)
        {
            if (!arePropertiesSet)
            {
                setProperties();
                arePropertiesSet = true;
            }
            else
            {
                animate();
            }
        }

        private void animate()
        {
            i += 1;
            foreach (FrameworkElement element in hostPanel.Children)
            {
                var elementProperty = element.Tag as ElementProperties;

                if (elementProperty == null) continue;
                var currentX = Canvas.GetLeft(element);
                var currentY = Canvas.GetTop(element);

                var setX = elementProperty.CurrentPosition.X;
                var setY = elementProperty.CurrentPosition.Y;

                var futureX = elementProperty.RandomPosition.X;
                var futureY = elementProperty.RandomPosition.Y;

                var dX = futureX - setX;
                var dY = futureY - setY;

                //var totalDistance = Math.Sqrt(Math.Pow(dX, 2) + Math.Pow(dY, 2));

                if (inDestinationRegion(currentX, futureX) || inDestinationRegion(currentY, futureY))
                {
                    setRandomCoordinates(element);
                }
                else
                {
                    Canvas.SetLeft(element, currentX + dX / 100);
                    Canvas.SetTop(element, currentY + dY / 100);
                }
            }
        }

        private static bool inDestinationRegion(double current, double future)
        {
            if (Math.Abs(current - future) < 2)
            {
                return true;
            }
            return false;
        }

        private void setRandomCoordinates(FrameworkElement element)
        {
            var elementProperty = element.Tag as ElementProperties;

            parentWindowSize = new Point(hostPanel.ActualWidth, hostPanel.ActualHeight);

            if (elementProperty == null) return;
            elementProperty.CurrentPosition = new Point(Canvas.GetLeft(element), Canvas.GetTop(element));
            elementProperty.RandomPosition = new Point(ranNum.Next((int)-element.ActualWidth,
                                                        (int)(parentWindowSize.X + element.ActualWidth)), ranNum.Next((int)-element.ActualHeight,
                                                        (int)(parentWindowSize.Y + element.ActualHeight)));
        }

        /// <summary>
        /// Element properties
        /// </summary>
        public class ElementProperties
        {
            /// <summary>
            /// Gets or sets the radius.
            /// </summary>
            /// <value>The radius.</value>
            public double Radius
            {
                get;
                set;
            }

            /// <summary>
            /// Gets or sets the speed.
            /// </summary>
            /// <value>The speed.</value>
            public double Speed
            {
                get;
                set;
            }

            /// <summary>
            /// Gets or sets the current position.
            /// </summary>
            /// <value>The current position.</value>
            public Point CurrentPosition
            {
                get;
                set;
            }

            /// <summary>
            /// Gets or sets the random position.
            /// </summary>
            /// <value>The random position.</value>
            public Point RandomPosition
            {
                get;
                set;
            }
        }
    }
}