﻿using System;
using System.Windows;
using System.Windows.Interactivity;
using System.Windows.Media;
using System.ComponentModel;

namespace Od.Behaviors.Animation
{
    /// <summary>
    /// Shake behavior
    /// Debug + Cleanup + new props + detach : OD 2010
    /// </summary>
    public class ShakeBehavior : Behavior<UIElement>
    {

        #region Declaration
        private UIElement attachedElement;
        private static readonly Random ranNum = new Random();
        private int currentAngle;
        private int currentDistant;
        int i;
        #endregion


        #region Property to Expose
        /// <summary>
        /// Gets or sets the max rotation angle.
        /// </summary>
        /// <value>The max rotation angle.</value>
        [Category("Animation Properties")]
        public int MaxRotationAngle
        {
            get { return (int)GetValue(MaxRotationAngleProperty); }
            set { SetValue(MaxRotationAngleProperty, value); }
        }

        /// <summary>
        /// Gets or sets the max rotation angle.
        /// </summary>
        public static readonly DependencyProperty MaxRotationAngleProperty = DependencyProperty.Register("Maximum Rotation Angle", typeof(int), typeof(ShakeBehavior), new PropertyMetadata(1));

        /// <summary>
        /// Gets or sets the max movement distant.
        /// </summary>
        /// <value>The max movement distant.</value>
        [Category("Animation Properties")]
        public int MaxMovementDistant
        {
            get { return (int)GetValue(MaxMovementDistantProperty); }
            set { SetValue(MaxMovementDistantProperty, value); }
        }

        /// <summary>
        /// Gets or sets the max movement distant.
        /// </summary>
        public static readonly DependencyProperty MaxMovementDistantProperty = DependencyProperty.Register("Maximum Movement Distant", typeof(int), typeof(ShakeBehavior), new PropertyMetadata(3));


        #region Frequency Property

        /// <summary>
        /// Gets or sets the frequency.
        /// </summary>
        /// <value>The frequency.</value>
        [Category("Animation Properties")]
        public double Frequency
        {
            get { return (double)GetValue(FrequencyProperty); }
            set { SetValue(FrequencyProperty, value); }
        }

        /// <summary>
        /// Gets or sets the frequency
        /// </summary>
        public static readonly DependencyProperty FrequencyProperty =
            DependencyProperty.Register(
                "Frequency", typeof(double), typeof(ShakeBehavior),
                new PropertyMetadata(10d, onFrequencyChanged));

        private static void onFrequencyChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            var sb = d as ShakeBehavior;
            if (sb != null)
            {
                if (sb.Frequency < 0d) sb.Frequency = 1d;
                if (sb.Frequency > 1000d) sb.Frequency = 1000d;
                sb.Frequency = ((int)(sb.Frequency * 1000d)) / 1000d;
            }
        }

        #endregion

        #region IsShaking Property 

        /// <summary>
        /// Gets or sets a value indicating whether this instance is shaking.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is shaking; otherwise, <c>false</c>.
        /// </value>
        public bool IsShaking
        {
            get { return (bool) GetValue(IsShakingProperty); }
            set { SetValue(IsShakingProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether this instance is shaking.
        /// </summary>
        public static readonly DependencyProperty IsShakingProperty =
            DependencyProperty.Register(
                "IsShaking", typeof (bool), typeof (ShakeBehavior),
                new PropertyMetadata(true, onIsShakingChanged));

        private static void onIsShakingChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {}

        #endregion


        #endregion

        /// <summary>
        /// Called after the behavior is attached to an AssociatedObject.
        /// </summary>
        /// <remarks>Override this to hook up functionality to the AssociatedObject.</remarks>
        protected override void OnAttached()
        {
            base.OnAttached();
            attachedElement = AssociatedObject;

            if (!DesignerProperties.GetIsInDesignMode(this))
            {
                CompositionTarget.Rendering += compositionTarget_Rendering;
            }
        }

        /// <summary>
        /// Called when the behavior is being detached from its AssociatedObject, but before it has actually occurred.
        /// </summary>
        /// <remarks>Override this to unhook functionality from the AssociatedObject.</remarks>
        protected override void OnDetaching()
        {
            if (!DesignerProperties.GetIsInDesignMode(this))
            {
                CompositionTarget.Rendering -= compositionTarget_Rendering;
                attachedElement.RenderTransform = null;
            }
            base.OnDetaching();
        }

        private void compositionTarget_Rendering(object sender, EventArgs e)
        {
            if (!IsShaking) return;
            i++;
            if (i % Frequency != 0) return;
            var rt = new TransformGroup();
            var newAngle = new RotateTransform();
            var newPosition = new TranslateTransform();

            GeneralTransform objGeneralTransform;
            try
            {
                objGeneralTransform = attachedElement.TransformToVisual(Application.Current.RootVisual);
            }
            catch
            {
                return;
            }
            var point = objGeneralTransform.Transform(new Point(0, 0));


            var oldTransform = attachedElement.RenderTransform;
            if (oldTransform != null)
            {
                //rt.Children.Add(oldTransform);
            }

            MaxRotationAngle = MaxRotationAngle % 360;
            var deltaAngle = ranNum.Next(-MaxRotationAngle - currentAngle, (MaxRotationAngle + 1) - currentAngle);
            newAngle.Angle = deltaAngle;
            currentAngle = currentAngle + deltaAngle;
            var deltaDistant = ranNum.Next(-MaxMovementDistant - currentDistant, (MaxMovementDistant + 1) - currentDistant);
            newPosition.X = deltaDistant;
            currentDistant = currentDistant + deltaDistant;
            newAngle.CenterX = point.X + attachedElement.RenderSize.Width / 2;
            newAngle.CenterY = point.Y + attachedElement.RenderSize.Height / 2;

            rt.Children.Add(newAngle);
            rt.Children.Add(newPosition);
            var mt = new MatrixTransform { Matrix = rt.Value };

            attachedElement.RenderTransform = mt;
        }

    }
}
