﻿using System.ComponentModel;
using System.Windows;
using System.Windows.Interactivity;
using System.Windows.Media.Animation;
using System.Windows.Media;
using System;

namespace Od.Behaviors.Animation
{
    /// <summary>
    /// Show / hide Flip animation
    /// Debug + cleanup + correct reverse animation + CLR props were not connected to DP props : OD 2010
    /// </summary>
    public class ShowHideFlipAction : TriggerAction<FrameworkElement>
    {
        private Storyboard mainStoryboard;

        #region Public Properties

        #region "Forward"

        /// <summary>
        /// Play forward or backward
        /// </summary>
        public static readonly DependencyProperty ForwardProperty = DependencyProperty.Register("Forward", typeof(bool), typeof(ShowHideFlipAction), new PropertyMetadata(true));

        /// <summary>
        /// Gets or sets a value indicating whether this <see cref="ShowHideFlipAction"/> is forward.
        /// </summary>
        /// <value><c>true</c> if forward; otherwise, <c>false</c>.</value>
        [Category("ShowHideFlip Properties")]
        public bool Forward
        {
            get { return (bool)GetValue(ForwardProperty); }
            set { SetValue(ForwardProperty, value); }
        }

        #endregion

        #region "ShowElementName"

        /// <summary>
        /// Gets or sets the name of the show element (Face 1)
        /// </summary>
        public static readonly DependencyProperty ShowElementNameProperty =
            DependencyProperty.Register("ShowElementName", typeof(string), typeof(ShowHideFlipAction), new PropertyMetadata(null));

        /// <summary>
        /// Gets or sets the name of the show element (Face 1)
        /// </summary>
        /// <value>The name of the show element.</value>
        [Category("ShowHideFlip Properties")]
        public string ShowElementName
        {
            get { return (string) GetValue(ShowElementNameProperty); }
            set { SetValue(ShowElementNameProperty, value); }
        }

        #endregion

        #region "HideElementName"

        /// <summary>
        /// 
        /// </summary>
        public static readonly DependencyProperty HideElementNameProperty =
            DependencyProperty.Register("HideElementName", typeof(string), typeof(ShowHideFlipAction), new PropertyMetadata(null));

        /// <summary>
        /// Gets or sets the name of the hide element.
        /// </summary>
        /// <value>The name of the hide element.</value>
        [Category("ShowHideFlip Properties")]
        public string HideElementName
        {
            get { return (string)GetValue(HideElementNameProperty); }
            set { SetValue(HideElementNameProperty, value); }
        }

        #endregion

        #endregion

        /// <summary>
        /// Invokes the action.
        /// </summary>
        /// <param name="parameter">The parameter to the action. If the action does not require a parameter, the parameter may be set to a <c>null</c> reference.</param>
        protected override void Invoke(object parameter)
        {
            if (AssociatedObject == null) return;
            UIElement showElement;
            UIElement hideElement;
            var parent = AssociatedObject;
            if (onePass)
            {
                hideElement = parent.FindName(ShowElementName) as UIElement;
                showElement = parent.FindName(HideElementName) as UIElement;
            }
            else
            {
                showElement = parent.FindName(ShowElementName) as UIElement;
                hideElement = parent.FindName(HideElementName) as UIElement;
            }


            if (showElement == null || hideElement == null) return;

            if (showElement.Projection == null)
            {
                showElement.Projection = new PlaneProjection();
                showElement.RenderTransformOrigin = new Point(.5, .5);
            }

            if (hideElement.Projection == null)
            {
                hideElement.Projection = new PlaneProjection();
                hideElement.RenderTransformOrigin = new Point(.5, .5);
            }

            mainStoryboard = new Storyboard();

            // Show           
            var showPp = new PlaneProjection();
            var showAnimation = new DoubleAnimationUsingKeyFrames { BeginTime = new TimeSpan(0) };
            if (Forward)
            {
                showAnimation.KeyFrames.Add(new EasingDoubleKeyFrame { KeyTime = new TimeSpan(0), Value = -90 });
                showAnimation.KeyFrames.Add(new EasingDoubleKeyFrame { KeyTime = new TimeSpan(0, 0, 0, 0, 400), Value = -90 });
                showAnimation.KeyFrames.Add(new EasingDoubleKeyFrame { KeyTime = new TimeSpan(0, 0, 0, 0, 600), Value = 0 });
            }
            else
            {
                showAnimation.KeyFrames.Add(new EasingDoubleKeyFrame { KeyTime = new TimeSpan(0), Value = 90 });
                showAnimation.KeyFrames.Add(new EasingDoubleKeyFrame { KeyTime = new TimeSpan(0, 0, 0, 0, 400), Value = 90 });
                showAnimation.KeyFrames.Add(new EasingDoubleKeyFrame { KeyTime = new TimeSpan(0, 0, 0, 0, 600), Value = 0 });
            }

            Storyboard.SetTargetProperty(showAnimation, new PropertyPath("RotationY"));
            Storyboard.SetTarget(showAnimation, showPp);
            showElement.Projection = showPp;

            var showAnimationVisibility = new ObjectAnimationUsingKeyFrames { BeginTime = new TimeSpan(0) };
            showAnimationVisibility.KeyFrames.Add(new DiscreteObjectKeyFrame { KeyTime = new TimeSpan(0), Value = Visibility.Visible });
            Storyboard.SetTargetProperty(showAnimationVisibility, new PropertyPath("Visibility"));
            Storyboard.SetTarget(showAnimationVisibility, showElement);

            // Hide            
            var hidePp = new PlaneProjection();
            var hideAnimation = new DoubleAnimationUsingKeyFrames { BeginTime = new TimeSpan(0) };
            if (Forward)
            {
                hideAnimation.KeyFrames.Add(new EasingDoubleKeyFrame { KeyTime = new TimeSpan(0), Value = 0 });
                hideAnimation.KeyFrames.Add(new EasingDoubleKeyFrame { KeyTime = new TimeSpan(0, 0, 0, 0, 400), Value = 90 });
            }
            else
            {
                hideAnimation.KeyFrames.Add(new EasingDoubleKeyFrame { KeyTime = new TimeSpan(0), Value = 0 });
                hideAnimation.KeyFrames.Add(new EasingDoubleKeyFrame { KeyTime = new TimeSpan(0, 0, 0, 0, 400), Value = -90 });
            }
            Storyboard.SetTargetProperty(hideAnimation, new PropertyPath("RotationY"));
            Storyboard.SetTarget(hideAnimation, hidePp);
            hideElement.Projection = hidePp;

            var hideAnimationVisibility = new ObjectAnimationUsingKeyFrames { BeginTime = new TimeSpan(0) };
            hideAnimationVisibility.KeyFrames.Add(new DiscreteObjectKeyFrame { KeyTime = new TimeSpan(0, 0, 0, 0, 400), Value = Visibility.Collapsed });
            Storyboard.SetTargetProperty(hideAnimationVisibility, new PropertyPath("Visibility"));
            Storyboard.SetTarget(hideAnimationVisibility, hideElement);

            // Wire up
            mainStoryboard.Children.Add(showAnimation);
            mainStoryboard.Children.Add(showAnimationVisibility);
            mainStoryboard.Children.Add(hideAnimation);
            mainStoryboard.Children.Add(hideAnimationVisibility);

            mainStoryboard.Duration = new TimeSpan(0, 0, 0, 0, 600);
            mainStoryboard.Completed += mainStoryboard_Completed;
            mainStoryboard.Begin();

        }

        private bool onePass;

        void mainStoryboard_Completed(object sender, EventArgs e)
        {
            onePass = !onePass;
        }

    }
}