﻿using System;
using System.ComponentModel;
using System.Windows;
using System.Windows.Interactivity;
using System.Windows.Media;
using System.Windows.Media.Animation;

namespace Od.Behaviors.Animation
{

    /// <summary>
    /// Swivel rotation
    /// </summary>
    public enum SwivelRotationDirection
    {
        LeftToRight,
        RightToLeft,
        TopToBottom,
        BottomToTop
    }

    /// <summary>
    /// Data for swivel rotation
    /// </summary>
    public class SwivelRotationData
    {
        /// <summary>
        /// Gets or sets from degrees.
        /// </summary>
        /// <value>From degrees.</value>
        public double FromDegrees { get; set; }
        /// <summary>
        /// Gets or sets the mid degrees.
        /// </summary>
        /// <value>The mid degrees.</value>
        public double MidDegrees { get; set; }
        /// <summary>
        /// Gets or sets to degrees.
        /// </summary>
        /// <value>To degrees.</value>
        public double ToDegrees { get; set; }
        /// <summary>
        /// Gets or sets the rotation property.
        /// </summary>
        /// <value>The rotation property.</value>
        public string RotationProperty { get; set; }
        /// <summary>
        /// Gets or sets the plane projection.
        /// </summary>
        /// <value>The plane projection.</value>
        public PlaneProjection PlaneProjection { get; set; }
        /// <summary>
        /// Gets or sets the duration of the animation.
        /// </summary>
        /// <value>The duration of the animation.</value>
        public Duration AnimationDuration { get; set; }
    }

    /// <summary>
    /// Swivel animation action
    /// Debug + cleanup + CRL props were not connected to DP props + automatic duration threw exception : OD 2010
    /// </summary>
    public class SwivelAction : TriggerAction<FrameworkElement>
    {
        #region Front element

        /// <summary>
        ///  Gets or sets the name of the front element.
        /// </summary>
        public static readonly DependencyProperty FrontElementNameProperty =
            DependencyProperty.Register("FrontElementName", typeof(string),
                                        typeof(SwivelAction), new PropertyMetadata(null));

        /// <summary>
        /// Gets or sets the name of the front element.
        /// </summary>
        /// <value>The name of the front element.</value>
        [Category("Swivel Properties")]
        public string FrontElementName
        {
            get { return (string)GetValue(FrontElementNameProperty); }
            set { SetValue(FrontElementNameProperty, value); }
        }

        #endregion

        #region Back element

        /// <summary>
        /// Gets or sets the name of the back element.
        /// </summary>
        public static readonly DependencyProperty BackElementNameProperty =
            DependencyProperty.Register("BackElementName", typeof(string),
                                        typeof(SwivelAction), new PropertyMetadata(null));

        /// <summary>
        /// Gets or sets the name of the back element.
        /// </summary>
        /// <value>The name of the back element.</value>
        [Category("Swivel Properties")]
        public string BackElementName
        {
            get { return (string)GetValue(BackElementNameProperty); }
            set { SetValue(BackElementNameProperty, value); }
        }

        #endregion

        #region Duration

        /// <summary>
        /// Gets or sets the duration.
        /// </summary>
        public static readonly DependencyProperty DurationProperty =
            DependencyProperty.Register("Duration", typeof(Duration),
                                        typeof(SwivelAction), new PropertyMetadata(null));

        /// <summary>
        /// Gets or sets the duration.
        /// </summary>
        /// <value>The duration.</value>
        [Category("Animation Properties")]
        public Duration Duration
        {
            get { return (Duration)GetValue(DurationProperty); }
            set { SetValue(DurationProperty, value); }
        }

        #endregion

        #region Rotation Direction

        /// <summary>
        /// Gets or sets the rotation.
        /// </summary>
        public static readonly DependencyProperty RotationProperty =
            DependencyProperty.Register("Rotation", typeof(SwivelRotationDirection),
                                        typeof(SwivelAction), new PropertyMetadata(SwivelRotationDirection.LeftToRight));

        /// <summary>
        /// Gets or sets the rotation.
        /// </summary>
        /// <value>The rotation.</value>
        [Category("Animation Properties")]
        public SwivelRotationDirection Rotation
        {
            get { return (SwivelRotationDirection)GetValue(RotationProperty); }
            set { SetValue(RotationProperty, value); }
        }

        #endregion

        private readonly Storyboard frontToBackStoryboard = new Storyboard();
        private readonly Storyboard backToFrontStoryboard = new Storyboard();
        private bool forward = true;

        /// <summary>
        /// Invokes the action.
        /// </summary>
        /// <param name="parameter">The parameter to the action. If the action does not require a parameter, the parameter may be set to a <c>null</c> reference.</param>
        protected override void Invoke(object parameter)
        {
            if (AssociatedObject == null) return;

            var duration = !Duration.HasTimeSpan
                               ? new Duration(new TimeSpan(0, 0, 0, 0, 200))
                               : Duration;

            var parent = AssociatedObject;

            var front = parent.FindName(FrontElementName) as UIElement;
            var back = parent.FindName(BackElementName) as UIElement;
            if (front == null || back == null) return;

            if (front.Projection == null || back.Projection == null)
            {
                front.Projection = new PlaneProjection();
                front.RenderTransformOrigin = new Point(.5, .5);
                front.Visibility = Visibility.Visible;

                back.Projection = new PlaneProjection { CenterOfRotationY = .5, RotationY = 180.0 };
                back.RenderTransformOrigin = new Point(.5, .5);
                back.Visibility = Visibility.Collapsed;

                SwivelRotationData showBackRotation = null;
                SwivelRotationData hideFrontRotation = null;
                SwivelRotationData showFrontRotation = null;
                SwivelRotationData hideBackRotation = null;

                var frontPp = new PlaneProjection();
                var backPp = new PlaneProjection();

                switch (Rotation)
                {
                    case SwivelRotationDirection.LeftToRight:
                        backPp.CenterOfRotationY = frontPp.CenterOfRotationY = 0.5;
                        showBackRotation = new SwivelRotationData { FromDegrees = 180.0, MidDegrees = 90.0, ToDegrees = 0.0, RotationProperty = "RotationY", PlaneProjection = backPp, AnimationDuration = duration };
                        hideFrontRotation = new SwivelRotationData { FromDegrees = 0.0, MidDegrees = -90.0, ToDegrees = -180.0, RotationProperty = "RotationY", PlaneProjection = frontPp, AnimationDuration = duration };
                        showFrontRotation = new SwivelRotationData { FromDegrees = -180.0, MidDegrees = -90.0, ToDegrees = 0.0, RotationProperty = "RotationY", PlaneProjection = frontPp, AnimationDuration = duration };
                        hideBackRotation = new SwivelRotationData { FromDegrees = 0.0, MidDegrees = 90.0, ToDegrees = 180.0, RotationProperty = "RotationY", PlaneProjection = backPp, AnimationDuration = duration };
                        break;
                    case SwivelRotationDirection.RightToLeft:
                        backPp.CenterOfRotationY = frontPp.CenterOfRotationY = 0.5;
                        showBackRotation = new SwivelRotationData { FromDegrees = -180.0, MidDegrees = -90.0, ToDegrees = 0.0, RotationProperty = "RotationY", PlaneProjection = backPp, AnimationDuration = duration };
                        hideFrontRotation = new SwivelRotationData { FromDegrees = 0.0, MidDegrees = 90.0, ToDegrees = 180.0, RotationProperty = "RotationY", PlaneProjection = frontPp, AnimationDuration = duration };
                        showFrontRotation = new SwivelRotationData { FromDegrees = 180.0, MidDegrees = 90.0, ToDegrees = 0.0, RotationProperty = "RotationY", PlaneProjection = frontPp, AnimationDuration = duration };
                        hideBackRotation = new SwivelRotationData { FromDegrees = 0.0, MidDegrees = -90.0, ToDegrees = -180.0, RotationProperty = "RotationY", PlaneProjection = backPp, AnimationDuration = duration };
                        break;
                    case SwivelRotationDirection.BottomToTop:
                        backPp.CenterOfRotationX = frontPp.CenterOfRotationX = 0.5;
                        showBackRotation = new SwivelRotationData { FromDegrees = 180.0, MidDegrees = 90.0, ToDegrees = 0.0, RotationProperty = "RotationX", PlaneProjection = backPp, AnimationDuration = duration };
                        hideFrontRotation = new SwivelRotationData { FromDegrees = 0.0, MidDegrees = -90.0, ToDegrees = -180.0, RotationProperty = "RotationX", PlaneProjection = frontPp, AnimationDuration = duration };
                        showFrontRotation = new SwivelRotationData { FromDegrees = -180.0, MidDegrees = -90.0, ToDegrees = 0.0, RotationProperty = "RotationX", PlaneProjection = frontPp, AnimationDuration = duration };
                        hideBackRotation = new SwivelRotationData { FromDegrees = 0.0, MidDegrees = 90.0, ToDegrees = 180.0, RotationProperty = "RotationX", PlaneProjection = backPp, AnimationDuration = duration };
                        break;
                    case SwivelRotationDirection.TopToBottom:
                        backPp.CenterOfRotationX = frontPp.CenterOfRotationX = 0.5;
                        showBackRotation = new SwivelRotationData { FromDegrees = -180.0, MidDegrees = -90.0, ToDegrees = 0.0, RotationProperty = "RotationX", PlaneProjection = backPp, AnimationDuration = duration };
                        hideFrontRotation = new SwivelRotationData { FromDegrees = 0.0, MidDegrees = 90.0, ToDegrees = 180.0, RotationProperty = "RotationX", PlaneProjection = frontPp, AnimationDuration = duration };
                        showFrontRotation = new SwivelRotationData { FromDegrees = 180.0, MidDegrees = 90.0, ToDegrees = 0.0, RotationProperty = "RotationX", PlaneProjection = frontPp, AnimationDuration = duration };
                        hideBackRotation = new SwivelRotationData { FromDegrees = 0.0, MidDegrees = -90.0, ToDegrees = -180.0, RotationProperty = "RotationX", PlaneProjection = backPp, AnimationDuration = duration };
                        break;
                }

                front.RenderTransformOrigin = new Point(.5, .5);
                back.RenderTransformOrigin = new Point(.5, .5);

                front.Projection = frontPp;
                back.Projection = backPp;

                frontToBackStoryboard.Duration = duration;
                backToFrontStoryboard.Duration = duration;

                // Rotation
                frontToBackStoryboard.Children.Add(createRotationAnimation(showBackRotation));
                frontToBackStoryboard.Children.Add(createRotationAnimation(hideFrontRotation));
                backToFrontStoryboard.Children.Add(createRotationAnimation(hideBackRotation));
                backToFrontStoryboard.Children.Add(createRotationAnimation(showFrontRotation));


                // Visibility
                if (showBackRotation != null)
                    frontToBackStoryboard.Children.Add(createVisibilityAnimation(showBackRotation.AnimationDuration, front, false));
                if (hideFrontRotation != null)
                    frontToBackStoryboard.Children.Add(createVisibilityAnimation(hideFrontRotation.AnimationDuration, back, true));
                if (hideBackRotation != null)
                    backToFrontStoryboard.Children.Add(createVisibilityAnimation(hideBackRotation.AnimationDuration, front, true));
                if (showFrontRotation != null)
                    backToFrontStoryboard.Children.Add(createVisibilityAnimation(showFrontRotation.AnimationDuration, back, false));
            }

            if (forward)
            {
                frontToBackStoryboard.Begin();
                forward = false;
            }
            else
            {
                backToFrontStoryboard.Begin();
                forward = true;
            }
        }

        private static ObjectAnimationUsingKeyFrames createVisibilityAnimation(Duration duration, DependencyObject element, bool show)
        {
            var animation = new ObjectAnimationUsingKeyFrames { BeginTime = new TimeSpan(0) };
            animation.KeyFrames.Add(new DiscreteObjectKeyFrame { KeyTime = new TimeSpan(0), Value = (show ? Visibility.Collapsed : Visibility.Visible) });
            animation.KeyFrames.Add(new DiscreteObjectKeyFrame { KeyTime = new TimeSpan(duration.TimeSpan.Ticks / 2), Value = (show ? Visibility.Visible : Visibility.Collapsed) });
            Storyboard.SetTargetProperty(animation, new PropertyPath("Visibility"));
            Storyboard.SetTarget(animation, element);
            return animation;
        }


        private static DoubleAnimationUsingKeyFrames createRotationAnimation(SwivelRotationData rd)
        {
            var animation = new DoubleAnimationUsingKeyFrames { BeginTime = new TimeSpan(0) };
            animation.KeyFrames.Add(new EasingDoubleKeyFrame { KeyTime = new TimeSpan(0), Value = rd.FromDegrees, EasingFunction = new CubicEase { EasingMode = EasingMode.EaseIn } });
            animation.KeyFrames.Add(new EasingDoubleKeyFrame { KeyTime = new TimeSpan(rd.AnimationDuration.TimeSpan.Ticks / 2), Value = rd.MidDegrees });
            animation.KeyFrames.Add(new EasingDoubleKeyFrame { KeyTime = new TimeSpan(rd.AnimationDuration.TimeSpan.Ticks), Value = rd.ToDegrees, EasingFunction = new CubicEase { EasingMode = EasingMode.EaseOut } });
            Storyboard.SetTargetProperty(animation, new PropertyPath(rd.RotationProperty));
            Storyboard.SetTarget(animation, rd.PlaneProjection);
            return animation;
        }
    }

}