﻿using System.Windows;
using System.Windows.Media;
using System.Windows.Interactivity;
using System.ComponentModel;

namespace Od.Behaviors.Appearance
{
    /// <summary>
    /// Clip to bound
    /// debug + cleanup + dp props : OD 2010
    /// </summary>
    public class ClipToBoundsBehavior : Behavior<FrameworkElement>
    {
        /// <summary>
        /// Called after the behavior is attached to an AssociatedObject.
        /// </summary>
        /// <remarks>Override this to hook up functionality to the AssociatedObject.</remarks>
        protected override void OnAttached()
        {
            base.OnAttached();

            AssociatedObject.Clip = clip(new Size(AssociatedObject.ActualWidth, AssociatedObject.ActualHeight));
            AssociatedObject.SizeChanged += associatedObject_SizeChanged;
        }
        /// <summary>
        /// Called when the behavior is being detached from its AssociatedObject, but before it has actually occurred.
        /// </summary>
        /// <remarks>Override this to unhook functionality from the AssociatedObject.</remarks>
        protected override void OnDetaching()
        {
            base.OnDetaching();

            AssociatedObject.SizeChanged -= associatedObject_SizeChanged;
            AssociatedObject.Clip = null;
        }

        private void associatedObject_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            AssociatedObject.Clip = clip(e.NewSize);
        }

        private Geometry clip(Size size)
        {
            if (IsInverted)
            {
                var gg = new GeometryGroup { FillRule = FillRule.EvenOdd };
                gg.Children.Add(new RectangleGeometry { Rect = new Rect(-15, -15, size.Width + 30, size.Height + 30), RadiusX = RadiusX, RadiusY = RadiusY });
                gg.Children.Add(new RectangleGeometry { Rect = new Rect(0, 0, size.Width, size.Height), RadiusX = RadiusX, RadiusY = RadiusY });

                return gg;
            }
            return new RectangleGeometry { Rect = new Rect(0, 0, size.Width, size.Height), RadiusX = RadiusX, RadiusY = RadiusY };
        }

        #region RadiusX Property 

        /// <summary>
        /// Gets or sets the radius X.
        /// </summary>
        /// <value>The radius X.</value>
        [Category("Common Properties")]
        public double RadiusX
        {
            get { return (double) GetValue(RadiusXProperty); }
            set { SetValue(RadiusXProperty, value); }
        }

        public static readonly DependencyProperty RadiusXProperty =
            DependencyProperty.Register(
                "RadiusX", typeof (double), typeof (ClipToBoundsBehavior),
                new PropertyMetadata(0d, OnRadiusXChanged));

        private static void OnRadiusXChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {}

        #endregion

        #region RadiusY Property 
        /// <summary>
        /// Gets or sets the radius Y.
        /// </summary>
        /// <value>The radius Y.</value>
        [Category("Common Properties")]
        public double RadiusY
        {
            get { return (double) GetValue(RadiusYProperty); }
            set { SetValue(RadiusYProperty, value); }
        }

        public static readonly DependencyProperty RadiusYProperty =
            DependencyProperty.Register(
                "RadiusY", typeof (double), typeof (ClipToBoundsBehavior),
                new PropertyMetadata(0d, OnRadiusYChanged));

        private static void OnRadiusYChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {}

        #endregion

        #region IsInverted Property 

        /// <summary>
        /// Gets or sets a value indicating whether the clipping region is inverted.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is inverted; otherwise, <c>false</c>.
        /// </value>
        [Category("Common Properties")]
        public bool IsInverted
        {
            get { return (bool) GetValue(IsInvertedProperty); }
            set { SetValue(IsInvertedProperty, value); }
        }

        public static readonly DependencyProperty IsInvertedProperty =
            DependencyProperty.Register(
                "IsInverted", typeof (bool), typeof (ClipToBoundsBehavior),
                new PropertyMetadata(false, OnIsInvertedChanged));

        private static void OnIsInvertedChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        { }

        #endregion

        
    }
}