﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Interactivity;
using System.Windows.Media.Animation;

namespace Od.Behaviors.Appearance
{
    /// <summary>
    /// Transparency behavior (change opacity when mouse is outside the object)
    /// Cleanup : OD 2010
    /// </summary>
    public class TransparencyBehavior : Behavior<FrameworkElement>
    {
        Storyboard storyboard;
        DoubleAnimation animation;

        /// <summary>
        /// Gets or sets the inactive transparency.
        /// </summary>
        public static readonly DependencyProperty InactiveTransparencyProperty = 
            DependencyProperty.Register("InactiveTransparency", typeof(double), typeof(TransparencyBehavior), new PropertyMetadata(0.5));
        /// <summary>
        /// Gets or sets the inactive transparency.
        /// </summary>
        /// <value>The inactive transparency.</value>
        public double InactiveTransparency
        {
            get { return (double)GetValue(InactiveTransparencyProperty); }
            set { SetValue(InactiveTransparencyProperty, value); }
        }

        /// <summary>
        ///  Gets or sets the anim duration
        /// </summary>
        public static readonly DependencyProperty DurationProperty = 
            DependencyProperty.Register("Duration", typeof(TimeSpan), typeof(TransparencyBehavior), new PropertyMetadata(TimeSpan.FromSeconds(0.2)));
        /// <summary>
        /// Gets or sets the anim duration.
        /// </summary>
        /// <value>The duration.</value>
        public TimeSpan Duration
        {
            get { return (TimeSpan)GetValue(DurationProperty); }
            set { SetValue(DurationProperty, value); }
        }

        /// <summary>
        /// Called after the behavior is attached to an AssociatedObject.
        /// </summary>
        /// <remarks>Override this to hook up functionality to the AssociatedObject.</remarks>
        protected override void OnAttached()
        {
            base.OnAttached();

            storyboard = new Storyboard();
            animation = new DoubleAnimation {Duration = Duration};
            Storyboard.SetTarget(animation, AssociatedObject);
            Storyboard.SetTargetProperty(animation, new PropertyPath(UIElement.OpacityProperty));
            storyboard.Children.Add(animation);

            AssociatedObject.Opacity = InactiveTransparency;
            AssociatedObject.MouseEnter += onMouseEnter;
            AssociatedObject.MouseLeave += onMouseLeave;
        }

        /// <summary>
        /// Called when the behavior is being detached from its AssociatedObject, but before it has actually occurred.
        /// </summary>
        /// <remarks>Override this to unhook functionality from the AssociatedObject.</remarks>
        protected override void OnDetaching()
        {
            base.OnDetaching();

            AssociatedObject.MouseEnter -= onMouseEnter;
            AssociatedObject.MouseLeave -= onMouseLeave;
        }

        /// <summary>
        /// Called when [mouse leave].
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.Windows.Input.MouseEventArgs"/> instance containing the event data.</param>
        void onMouseLeave(object sender, MouseEventArgs e)
        {
            animation.To = InactiveTransparency;
            storyboard.Begin();
        }

        /// <summary>
        /// Called when [mouse enter].
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.Windows.Input.MouseEventArgs"/> instance containing the event data.</param>
        void onMouseEnter(object sender, MouseEventArgs e)
        {
            animation.To = 1;
            storyboard.Begin();
        }
    }
}