﻿using System;
using System.ComponentModel;
using System.Windows;
using System.Windows.Interactivity;
using System.Windows.Media;
using System.Windows.Media.Animation;

namespace Od.Behaviors.Effects
{
    public enum RotationDirection
    {
        LeftToRight,
        RightToLeft,
        TopToBottom,
        BottomToTop
    }

    public class FlipAction : TargetedTriggerAction<FrameworkElement>
    {
        public static readonly DependencyProperty FrontElementNameProperty =
            DependencyProperty.Register("FrontElementName", typeof(string),
                                        typeof(FlipAction), new PropertyMetadata(string.Empty,onFrontElementChanged));

        private static void onFrontElementChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            ((FlipAction)d).buildStoryboards();
        }

        [Category("Flip Properties")]
        public string FrontElementName
        {
            get
            {
                return (string)GetValue(FrontElementNameProperty);
            }
            set
            {
                SetValue(FrontElementNameProperty, value);
            }
        }

        public static readonly DependencyProperty BackElementNameProperty =
            DependencyProperty.Register("BackElementName", typeof(string),
                                        typeof(FlipAction), new PropertyMetadata(string.Empty, onBackElementChanged));

        private static void onBackElementChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            ((FlipAction)d).buildStoryboards();
        }

        [Category("Flip Properties")]
        public string BackElementName
        {
            get
            {
                return (string)GetValue(BackElementNameProperty);
            }
            set
            {
                SetValue(BackElementNameProperty, value);
            }
        }

        public static readonly DependencyProperty DurationProperty =
            DependencyProperty.Register("Duration", typeof(Duration),
                                        typeof(FlipAction), new PropertyMetadata(new Duration(TimeSpan.FromMilliseconds(500)),onDurationChanged));

        private static void onDurationChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            ((FlipAction)d).buildStoryboards();
        }

        [Category("Animation Properties")]
        public Duration Duration
        {
            get
            {
                return (Duration)GetValue(DurationProperty);
            }
            set
            {
                SetValue(DurationProperty, value);
            }
        }

        public static readonly DependencyProperty RotationProperty =
            DependencyProperty.Register("Rotation", typeof(RotationDirection),
                                        typeof(FlipAction), new PropertyMetadata(RotationDirection.LeftToRight,onRotationChanged));

        private static void onRotationChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            ((FlipAction)d).buildStoryboards();
        }

        [Category("Animation Properties")]
        public RotationDirection Rotation
        {
            get
            {
                return (RotationDirection)GetValue(RotationProperty);
            }
            set
            {
                SetValue(RotationProperty, value);
            }
        }

        public static readonly DependencyProperty FrontStoryboardProperty =
            DependencyProperty.Register("FrontStoryboard", typeof(Storyboard), typeof(FlipAction), null);

        public Storyboard FrontStoryBoard
        {
            get
            {
                return (Storyboard)GetValue(FrontStoryboardProperty);
            }
        }
        public static readonly DependencyProperty BackStoryboardProperty =
            DependencyProperty.Register("BackStoryboard", typeof(Storyboard), typeof(FlipAction), null);

        public Storyboard BackStoryboard
        {
            get
            {
                return (Storyboard)GetValue(BackStoryboardProperty);
            }
        }

        private bool forward = true;

        protected override void OnAttached()
        {
            base.OnAttached();
            var element = AssociatedObject as FrameworkElement;
            if (element != null) element.Loaded += TargetLoaded;
        }

        void TargetLoaded(object sender, RoutedEventArgs e)
        {
            buildStoryboards();
        }

        private void buildStoryboards()
        {
            var pp = Target.Projection as PlaneProjection;
            if (Target.Projection == null)
            {
                pp = new PlaneProjection { CenterOfRotationY = .51 };

                Target.RenderTransformOrigin = new Point(.5, .5);
                Target.Projection = pp;
            }

            var sbF = new Storyboard();
            var sbB = new Storyboard();

            var f = Target.FindName(FrontElementName) as UIElement;
            if (f != null)
            {
                var ppFront = new PlaneProjection { CenterOfRotationY = .51 };
                f.Projection = ppFront;
                f.RenderTransformOrigin = new Point(.5, .5);
            }
            var b = Target.FindName(BackElementName) as UIElement;
            if (b != null)
            {
                var ppBack = new PlaneProjection { CenterOfRotationY = .51, RotationY = 180.0 };
                b.Projection = ppBack;
                b.RenderTransformOrigin = new Point(.5, .5);
                b.Opacity = 0.0;
            }


            var to = 0d;
            var from = 180d;
            var property = "RotationY";

            switch (Rotation)
            {
                case RotationDirection.RightToLeft:
                    to = 180.0;
                    from = 0.0;
                    break;
                case RotationDirection.TopToBottom:
                    property = "RotationX";
                    break;
                case RotationDirection.BottomToTop:
                    to = 0.0;
                    from = 180.0;
                    property = "RotationX";
                    break;
            }

            sbF.Duration = Duration;
            sbB.Duration = Duration;

            sbF.Children.Add(CreateDoubleAnimation(pp, property, from, to, true));
            sbB.Children.Add(CreateDoubleAnimation(pp, property, to, from, true));

            sbF.Children.Add(CreateDoubleAnimation(f, "Opacity", 1.0, 0.0, false));
            sbB.Children.Add(CreateDoubleAnimation(f, "Opacity", 0.0, 1.0, false));

            sbF.Children.Add(CreateDoubleAnimation(b, "Opacity", 0.0, 1.0, false));
            sbB.Children.Add(CreateDoubleAnimation(b, "Opacity", 1.0, 0.0, false));

            SetValue(FrontStoryboardProperty, sbF);
            SetValue(BackStoryboardProperty, sbB);
        }

        protected override void Invoke(object parameter)
        {
            var sbF = GetValue(FrontStoryboardProperty) as Storyboard;
            var sbB = GetValue(BackStoryboardProperty) as Storyboard;

            if (forward)
            {
                if (sbF != null)
                {
                    sbF.Begin();
                    forward = false;
                }
            }
            else
            {
                if (sbB != null)
                {
                    sbB.Begin();
                    forward = true;
                }
            }
        }

        private static DoubleAnimation CreateDoubleAnimation(DependencyObject element, string property, double from,
                                                             double to, bool addEasing)
        {
            var da = new DoubleAnimation {To = to, From = from};
            if (addEasing)
                da.EasingFunction = new PowerEase { EasingMode = EasingMode.EaseOut, Power = 3 };


            Storyboard.SetTargetProperty(da, new PropertyPath(property));
            Storyboard.SetTarget(da, element);
            return da;
        }
    }
}
