﻿using System;
using System.ComponentModel;
using System.Windows;
using System.Windows.Input;
using System.Windows.Interactivity;
using System.Windows.Media;
using System.Windows.Media.Animation;

namespace Od.Behaviors.Effects
{
    [Description("Project2Mouse - A behavior that enables a projection over the X and Y angles in response to mouse movement.")]
    public class MouseProjectionAction : TargetedTriggerAction<FrameworkElement>
    {
        [Description("Angle animation duration (in milliseconds)")]
        [Category("Projection Animation Properties")]
        public int AnimationTime { get; set; }

        [Description("The Easing Function assigned to the animation")]
        [Category("Projection Animation Properties")]
        public IEasingFunction Easing { get; set; }

        [Description("The Positive/Negative X Angle amplitude for the perspective transform")]
        [Category("Projection Animation Properties")]
        public double AmplitudeAngleX { get; set; }

        [Description("The Positive/Negative Y Angle amplitude for the perspective transform")]
        [Category("Projection Animation Properties")]
        public double AmplitudeAngleY { get; set; }


        [Description("If the projection is oriented towards the mouse or against it.")]
        [Category("Projection Animation Properties")]
        public Boolean ProjectionTowardsMouse { get; set; }

        private TimeSpan tsAnimationTime;
        private FrameworkElement feSourceObject;
        private FrameworkElement feTargetObject;

        private double sourceWidth;
        private double sourceHeight;

        protected override void Invoke(object parameter)
        { }

        protected override void OnAttached()
        {
            base.OnAttached();
            feSourceObject = (FrameworkElement)AssociatedObject;
            tsAnimationTime = new TimeSpan(0, 0, 0, 0, AnimationTime);
            feSourceObject.Loaded += feSourceObject_Loaded;
        }

        private void DetermineSourceWidthAndHeight()
        {
            if (double.IsNaN(feSourceObject.ActualWidth) || feSourceObject.ActualWidth == 0)
            {
                sourceWidth = feSourceObject.Width;
            }
            else
            {
                sourceWidth = feSourceObject.ActualWidth;
            }

            if (double.IsNaN(feSourceObject.ActualHeight) || feSourceObject.ActualHeight == 0)
            {
                sourceHeight = feSourceObject.Height;
            }
            else
            {
                sourceHeight = feSourceObject.ActualHeight;
            }
        }

        void feSourceObject_Loaded(object sender, RoutedEventArgs e)
        {
            DetermineSourceWidthAndHeight();
            feSourceObject.MouseEnter += feSourceObject_MouseEnter;
            feSourceObject.MouseLeave += feSourceObject_MouseLeave;
            feSourceObject.MouseMove += feSourceObject_MouseMove;

            feTargetObject = (FrameworkElement)TargetObject;
            if (feTargetObject.Projection != null) return;
            feTargetObject.RenderTransformOrigin = new Point(0.5, 0.5);
            feTargetObject.Projection = new PlaneProjection();
        }

        protected override void OnDetaching()
        {
            base.OnDetaching();
            if (feSourceObject == null) return;
            feSourceObject.MouseEnter -= feSourceObject_MouseEnter;
            feSourceObject.MouseLeave -= feSourceObject_MouseLeave;
            feSourceObject.MouseMove -= feSourceObject_MouseMove;
        }

        void feSourceObject_MouseMove(object sender, MouseEventArgs e)
        {
            AnimateToAngleXY(GetDestinationAngleXY(e));
        }
        void feSourceObject_MouseLeave(object sender, MouseEventArgs e)
        {
            AnimateToAngleXY(new Point(0, 0));
        }

        void feSourceObject_MouseEnter(object sender, MouseEventArgs e)
        {
            AnimateToAngleXY(GetDestinationAngleXY(e));
        }

        private Point GetDestinationAngleXY(MouseEventArgs e)
        {
            if (((sourceHeight == 0) || (sourceWidth == 0)) || (Double.IsNaN(sourceHeight) || Double.IsNaN(sourceWidth)))
            {
                DetermineSourceWidthAndHeight();
            }

            double x;
            double y;
            var actualPos = e.GetPosition(feSourceObject);
            var pCenter = new Point(sourceWidth / 2, sourceHeight / 2);
            if (ProjectionTowardsMouse == false)
            {
                y = (actualPos.X - pCenter.X) / pCenter.X * AmplitudeAngleX;
                x = (pCenter.Y - actualPos.Y) / pCenter.Y * AmplitudeAngleY;
            }
            else
            {
                y = (pCenter.X - actualPos.X) / pCenter.X * AmplitudeAngleX;
                x = (actualPos.Y - pCenter.Y) / pCenter.Y * AmplitudeAngleY;
            }
            return new Point(x, y);
        }

        private void AnimateToAngleXY(Point xy)
        {
            playAnimation(feTargetObject, "(UIElement.Projection).(PlaneProjection.RotationX)", tsAnimationTime, xy.X, Easing);
            playAnimation(feTargetObject, "(UIElement.Projection).(PlaneProjection.RotationY)", tsAnimationTime, xy.Y, Easing);
        }

        public static void playAnimation(FrameworkElement element, string property, TimeSpan time, double value, IEasingFunction easingFunction)
        {
            var sb = new Storyboard();
            sb.Children.Clear();
            var animation = new DoubleAnimation {Duration = time, To = value, EasingFunction = easingFunction};
            Storyboard.SetTargetProperty(animation, new PropertyPath(property));
            Storyboard.SetTarget(animation, element);
            sb.Children.Add(animation);
            sb.Begin();
        }
    }
}
