﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Shapes;
using System.Windows.Interactivity;
using System.ComponentModel;

namespace Od.Behaviors.Effects
{
    public class RotateBehavior : Behavior<FrameworkElement>
    {
        private enum ControlParts
        {
            None,
            Center,
        }

        bool isRotating;
        ControlParts rotatePart;
        readonly CompositeTransform rotation = new CompositeTransform();
        Point objectCenter;

        #region  Properties

        #region Appareance Properties

        #region DragSpace

        public static readonly DependencyProperty DragSpaceProperty =
            DependencyProperty.Register("DragSpace", typeof(double), typeof(RotateBehavior),
            new PropertyMetadata(20.0, OnAppareanceElementChanged));

        /// <summary>
        /// Set the space from border for start rotate mode
        /// </summary>
        [Category("Appareance"), Description("Set the space from center for start rotate mode")]
        public double DragSpace
        {
            get { return (double)GetValue(DragSpaceProperty); }
            set { SetValue(DragSpaceProperty, value); }
        }

        #endregion

        #region DragIndicatorsFill

        public static readonly DependencyProperty DragIndicatorsFillProperty =
            DependencyProperty.Register("DragIndicatorsFill", typeof(Brush), typeof(RotateBehavior),
            new PropertyMetadata(new SolidColorBrush(Colors.Yellow), OnAppareanceElementChanged));

        /// <summary>
        /// Set the drags indicator BackGround
        /// </summary>
        [Category("Appareance"), Description("Set the drag indicator's Background")]
        public Brush DragIndicatorsFill
        {
            get { return (Brush)GetValue(DragIndicatorsFillProperty); }
            set { SetValue(DragIndicatorsFillProperty, value); }
        }

        #endregion

        #region DragIndicatorsOpacity

        public static readonly DependencyProperty DragIndicatorsOpacityProperty =
            DependencyProperty.Register("DragIndicatorsOpacity", typeof(double), typeof(RotateBehavior),
            new PropertyMetadata(0.8, OnAppareanceElementChanged));

        /// <summary>
        /// Set the drags indicator Opacity
        /// </summary>
        [Category("Appareance"), Description("Set the drag indicator's Opacity")]
        public Double DragIndicatorsOpacity
        {
            get { return (Double)GetValue(DragIndicatorsOpacityProperty); }
            set { SetValue(DragIndicatorsOpacityProperty, value); }
        }

        #endregion

        private static void OnAppareanceElementChanged(DependencyObject obj, DependencyPropertyChangedEventArgs e)
        {

            var source = (RotateBehavior)obj;

            if (source.AssociatedObject == null) return;

            source.setEllipse(source.rotatePart);
        }

        #endregion

        #region Comportment

        #region IsCenterRotationDraggable

        public static readonly DependencyProperty IsCenterRotationDraggableProperty =
            DependencyProperty.Register("IsCenterRotationDraggable", typeof(bool), typeof(RotateBehavior),
            new PropertyMetadata(true));

        /// <summary>
        /// Indicate if rotate dragging is possible
        /// </summary>
        [Category("Comportment"), Description("Enable/Disable Rotation")]
        public bool IsCenterRotationDraggable
        {
            get { return (bool)GetValue(IsCenterRotationDraggableProperty); }
            set { SetValue(IsCenterRotationDraggableProperty, value); }
        }

        #endregion

        #endregion

        #endregion

        #region behavior elements implementation

        protected override void OnAttached()
        {
            base.OnAttached();

            isRotating = false;
            rotatePart = ControlParts.None;
            setEllipse(ControlParts.None);

            AssociatedObject.MouseLeftButtonDown += AssociatedObject_MouseLeftButtonDown;
            AssociatedObject.MouseLeftButtonUp += AssociatedObject_MouseLeftButtonUp;
            AssociatedObject.MouseMove += AssociatedObject_MouseMove;
            AssociatedObject.MouseLeave += AssociatedObject_MouseLeave;

            // bind to this event because is possible retrive the parent only when associated object is loaded
            AssociatedObject.Loaded += AssociatedObject_Loaded;

        }



        protected override void OnDetaching()
        {
            base.OnDetaching();

            AssociatedObject.MouseLeftButtonDown -= AssociatedObject_MouseLeftButtonDown;
            AssociatedObject.MouseLeftButtonUp -= AssociatedObject_MouseLeftButtonUp;
            AssociatedObject.MouseMove -= AssociatedObject_MouseMove;
            AssociatedObject.MouseLeave -= AssociatedObject_MouseLeave;
        }

        #endregion


        #region Associated object events

        void AssociatedObject_MouseMove(object sender, MouseEventArgs e)
        {
            if (isRotating)
            {
                rotate(e.GetPosition((FrameworkElement)AssociatedObject.Parent));
                setEllipse(rotatePart);
            }
            else
            {
                setEllipse(getRotatePart(e.GetPosition(AssociatedObject)));
                setCursor(getRotatePart(e.GetPosition(AssociatedObject)));
            }
        }

        void AssociatedObject_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            AssociatedObject.ReleaseMouseCapture();
            isRotating = false;
            rotatePart = ControlParts.None;
            setEllipse(ControlParts.None);
        }

        void AssociatedObject_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            AssociatedObject.CaptureMouse();
            isRotating = true;
            rotatePart = getRotatePart(e.GetPosition(AssociatedObject));

            e.Handled = true;
        }

        void AssociatedObject_MouseLeave(object sender, MouseEventArgs e)
        {
            setEllipse(ControlParts.None);
        }


        void AssociatedObject_Loaded(object sender, RoutedEventArgs e)
        {
            AssociatedObject.Loaded -= AssociatedObject_Loaded;
        }

        #endregion


        #region rotating functions

        ControlParts getRotatePart(Point mousePosition)
        {
            var size = new Size(
                AssociatedObject.ActualWidth,
                AssociatedObject.ActualHeight);

            var center = new Point(size.Width / 2, size.Height / 2);
            objectCenter.Y = (double)AssociatedObject.GetValue(Canvas.TopProperty) + center.Y;
            objectCenter.X = (double)AssociatedObject.GetValue(Canvas.LeftProperty) + center.X;
            rotation.CenterX = center.X;
            rotation.CenterY = center.Y;

            if (mousePosition.X > center.X - DragSpace
                && mousePosition.X < center.X + DragSpace
                && mousePosition.Y > center.Y - DragSpace
                && mousePosition.Y < center.Y + DragSpace
                && IsCenterRotationDraggable)
                return ControlParts.Center;

            return ControlParts.None;
        }

        void setCursor(ControlParts part)
        {
            switch (part)
            {
                case ControlParts.None:
                    AssociatedObject.Cursor = Cursors.Arrow;
                    break;

                case ControlParts.Center:
                    AssociatedObject.Cursor = Cursors.Hand;
                    break;
            }

        }

        void setEllipse(ControlParts part)
        {
            var parent = AssociatedObject.Parent as Canvas;

            if (parent == null) return;

            for (var i = 0; i < parent.Children.Count; i++)
            {
                var e = parent.Children[i] as FrameworkElement;

                if (e == null) continue;

                if (e.Name != "__ROTATE_AUTO_GEN_ELLIPSE" + GetHashCode()) continue;
                e.Visibility = Visibility.Collapsed;
                parent.Children.RemoveAt(i);
                break;
            }

            var eli = new Ellipse
            {
                Name = "__ROTATE_AUTO_GEN_ELLIPSE" + GetHashCode(),
                Fill = DragIndicatorsFill,
                Opacity = DragIndicatorsOpacity
            };

            eli.SetValue(Canvas.ZIndexProperty, (int)AssociatedObject.GetValue(Canvas.ZIndexProperty) + 1);
            eli.IsHitTestVisible = false;

            switch (part)
            {
                case ControlParts.None:
                    eli = null;
                    break;

                case ControlParts.Center:
                    eli.Width = DragSpace;
                    eli.Height = DragSpace;
                    eli.SetValue(Canvas.TopProperty, (double)AssociatedObject.GetValue(Canvas.TopProperty) + (AssociatedObject.ActualHeight / 2 - DragSpace / 2));
                    eli.SetValue(Canvas.LeftProperty, (double)AssociatedObject.GetValue(Canvas.LeftProperty) + (AssociatedObject.ActualWidth / 2 - DragSpace / 2));
                    break;
            }

            if (eli == null) return;

            parent.Children.Add(eli);
        }

        void rotate(Point parentMousePosition)
        {
            AssociatedObject.RenderTransform = rotation;
            switch (rotatePart)
            {
                case ControlParts.Center:
                    rotation.Rotation = getAngle(parentMousePosition);
                    break;
            }
        }

        double getAngle(Point parentMouse)
        {
            var newRadAngle = -Math.Atan((objectCenter.X - parentMouse.X) / (objectCenter.Y - parentMouse.Y));
            var newDegAngle = toDegrees(newRadAngle);

            if (parentMouse.Y > objectCenter.Y)
                newDegAngle += 180;

            return newDegAngle;
        }

        static double toDegrees(double radians)
        {
            double degrees = radians * 180 / Math.PI;
            return degrees;
        }

        #endregion
    }
}