﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Interactivity;
using System.Windows.Media;
using System.Windows.Media.Animation;

namespace Od.Behaviors.Effects
{
    public class ShiverBehavior : TriggerAction<UIElement>
    {
        readonly TranslateTransform shiverTransform = new TranslateTransform();
        readonly Storyboard shiverStory = new Storyboard();
        readonly ElasticEase ease = new ElasticEase();
        readonly DoubleAnimation shiverAnimation = new DoubleAnimation();

        public ShiverBehavior()
        {
            // Set up ease
            ease.Springiness = 2;

            // Create the Animation
            shiverAnimation.EasingFunction = ease;
            Storyboard.SetTarget(shiverAnimation, shiverTransform);

            // Setup Variable Properties
            SetupProperties();

            // Attach it to the story
            shiverStory.Children.Add(shiverAnimation);
        }

        void SetupProperties()
        {
            ease.Oscillations = Oscillations;
            shiverAnimation.From = (double)0 - ShiverSize / 2;
            shiverAnimation.To = (double)ShiverSize / 2;
            shiverAnimation.Duration = ShiverDuration;
            Storyboard.SetTargetProperty(shiverAnimation, new PropertyPath(ShiverDirection == Orientation.Horizontal ? "X" : "Y"));
        }

        #region Oscillations (DependencyProperty)

        /// <summary>
        /// Number of Oscillations in the Behavior
        /// </summary>
        public int Oscillations
        {
            get { return (int)GetValue(OscillationsProperty); }
            set { SetValue(OscillationsProperty, value); }
        }
        public static readonly DependencyProperty OscillationsProperty =
            DependencyProperty.Register("Oscillations", typeof(int), typeof(ShiverBehavior),
            new PropertyMetadata(25, OnOscillationsChanged));

        private static void OnOscillationsChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            ((ShiverBehavior)d).OnOscillationsChanged(e);
        }

        protected virtual void OnOscillationsChanged(DependencyPropertyChangedEventArgs e)
        {
            SetupProperties();
        }
        #endregion


        #region ShiverSize (DependencyProperty)

        /// <summary>
        /// The depth of the shiver
        /// </summary>
        public int ShiverSize
        {
            get { return (int)GetValue(ShiverSizeProperty); }
            set { SetValue(ShiverSizeProperty, value); }
        }
        public static readonly DependencyProperty ShiverSizeProperty =
            DependencyProperty.Register("ShiverSize", typeof(int), typeof(ShiverBehavior),
            new PropertyMetadata(10, OnShiverSizeChanged));

        private static void OnShiverSizeChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            ((ShiverBehavior)d).OnShiverSizeChanged(e);
        }

        protected virtual void OnShiverSizeChanged(DependencyPropertyChangedEventArgs e)
        {
            SetupProperties();
        }
        #endregion


        #region ShiverDirection (DependencyProperty)

        /// <summary>
        /// Direction of the Shiver
        /// </summary>
        public Orientation ShiverDirection
        {
            get { return (Orientation)GetValue(ShiverDirectionProperty); }
            set { SetValue(ShiverDirectionProperty, value); }
        }
        public static readonly DependencyProperty ShiverDirectionProperty =
            DependencyProperty.Register("ShiverDirection", typeof(Orientation), typeof(ShiverBehavior),
            new PropertyMetadata(Orientation.Horizontal, OnShiverDirectionChanged));

        private static void OnShiverDirectionChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            ((ShiverBehavior)d).OnShiverDirectionChanged(e);
        }

        protected virtual void OnShiverDirectionChanged(DependencyPropertyChangedEventArgs e)
        {
            SetupProperties();
        }
        #endregion


        #region ShiverDuration (DependencyProperty)

        /// <summary>
        /// The duration of the Shiver
        /// </summary>  
        public TimeSpan ShiverDuration
        {
            get { return (TimeSpan)GetValue(ShiverDurationProperty); }
            set { SetValue(ShiverDurationProperty, value); }
        }
        public static readonly DependencyProperty ShiverDurationProperty =
            DependencyProperty.Register("ShiverDuration", typeof(TimeSpan), typeof(ShiverBehavior),
            new PropertyMetadata(TimeSpan.FromSeconds(2), OnShiverDurationChanged));

        private static void OnShiverDurationChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            ((ShiverBehavior)d).OnShiverDurationChanged(e);
        }

        protected virtual void OnShiverDurationChanged(DependencyPropertyChangedEventArgs e)
        {
            SetupProperties();
        }

        #endregion

        protected override void OnAttached()
        {
            base.OnAttached();

            // Attach the transform
            if (AssociatedObject.RenderTransform == null)
            {
                AssociatedObject.RenderTransform = shiverTransform;
            }
            else if (AssociatedObject.RenderTransform is TransformGroup)
            {
                var group = (TransformGroup)AssociatedObject.RenderTransform;
                group.Children.Add(shiverTransform);
            }
            else
            {
                var tx = AssociatedObject.RenderTransform;
                var group = new TransformGroup();
                group.Children.Add(tx);
                group.Children.Add(shiverTransform);
                AssociatedObject.RenderTransform = group;
            }
        }

        protected override void OnDetaching()
        {
            base.OnDetaching();

            // Attach the transform
            if (AssociatedObject.RenderTransform is TransformGroup)
            {
                ((TransformGroup)AssociatedObject.RenderTransform).Children.Remove(shiverTransform);
            }
            else
            {
                AssociatedObject.RenderTransform = null;
            }
        }

        protected override void Invoke(object parameter)
        {
            shiverStory.Begin();
        }

    }
}
