﻿using System;
using System.ComponentModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Interactivity;
using System.Windows.Media;
using System.Windows.Media.Animation;

namespace Od.Behaviors.Effects
{
    public class Slide3DToMouseAction : TargetedTriggerAction<FrameworkElement>
    {
        // properties
        [Description("Angle animation duration (in milliseconds)")]
        [Category("Projection Animation Properties")]
        public int AnimationTime { get; set; } // how long must the animation take

        [Description("The Easing Function assigned to the animation")]
        [Category("Projection Animation Properties")]
        public IEasingFunction Easing { get; set; }

        [Description("The Positive/Negative X Angle amplitude for the perspective transform")]
        [Category("Projection Animation Properties")]
        public double AmplitudeAngleX { get; set; }
        [Category("Slide Animation Properties")]
        public double AmplitudeSlideX { get; set; }

        [Description("The Positive/Negative Y Angle amplitude for the perspective transform")]
        [Category("Projection Animation Properties")]
        public double AmplitudeAngleY { get; set; }
        [Category("Slide Animation Properties")]
        public double AmplitudeSlideY { get; set; }

        [Description("If the projection is oriented towards the mouse or against it.")]
        [Category("Projection Animation Properties")]
        public Boolean ProjectionTowardsMouse { get; set; }

        private TimeSpan tsAnimationTime;
        private FrameworkElement feSourceObject;
        private FrameworkElement feTargetObject;
        private PlaneProjection projectionTargetObject;

        private double sourceWidth;
        private double sourceHeight;

        protected override void Invoke(object parameter)
        {
            
        }

        protected override void OnAttached()
        {
            base.OnAttached();
            feSourceObject = (FrameworkElement)AssociatedObject;
            tsAnimationTime = new TimeSpan(0, 0, 0, 0, AnimationTime);
            feSourceObject.Loaded += feSourceObject_Loaded;
        }

        private void DetermineSourceWidthAndHeight()
        {
            if (double.IsNaN(feSourceObject.ActualWidth) || feSourceObject.ActualWidth == 0)
            {
                sourceWidth = feSourceObject.Width;
            }
            else
            {
                sourceWidth = feSourceObject.ActualWidth;
            }

            if (double.IsNaN(feSourceObject.ActualHeight) || feSourceObject.ActualHeight == 0)
            {
                sourceHeight = feSourceObject.Height;
            }
            else
            {
                sourceHeight = feSourceObject.ActualHeight;
            }
        }
        private Point basePoint;
        void feSourceObject_Loaded(object sender, RoutedEventArgs e)
        {
            DetermineSourceWidthAndHeight();
            feSourceObject.MouseEnter += feSourceObject_MouseEnter;
            feSourceObject.MouseLeave += feSourceObject_MouseLeave;
            feSourceObject.MouseMove += feSourceObject_MouseMove;
            projectionTargetObject = new PlaneProjection();

            feTargetObject = (FrameworkElement)TargetObject;
            if (feTargetObject.Projection == null)
            {
                feTargetObject.RenderTransformOrigin = new Point(0.5, 0.5);
                feTargetObject.Projection = new PlaneProjection();
            }
            else
            {
                projectionTargetObject.RotationX = ((PlaneProjection)feTargetObject.Projection).RotationX;
                projectionTargetObject.RotationY = ((PlaneProjection)feTargetObject.Projection).RotationY;
            }
            basePoint.X = Canvas.GetLeft(feTargetObject);
            basePoint.Y = Canvas.GetTop(feTargetObject);
        }

        protected override void OnDetaching()
        {
            base.OnDetaching();
            if (feSourceObject == null) return;
            feSourceObject.MouseEnter -= feSourceObject_MouseEnter;
            feSourceObject.MouseLeave -= feSourceObject_MouseLeave;
            feSourceObject.MouseMove -= feSourceObject_MouseMove;
        }

        void feSourceObject_MouseMove(object sender, MouseEventArgs e)
        {
            if (feTargetObject.Visibility == Visibility.Collapsed)
                return;

            Point p = GetDestinationAngleXY(e);
            AnimateToAngleXY(p);
        }
        void feSourceObject_MouseLeave(object sender, MouseEventArgs e)
        {
            if (feTargetObject.Visibility == Visibility.Collapsed)
                return;

            AnimateToAngleXY(new Point(0, 0));
        }

        void feSourceObject_MouseEnter(object sender, MouseEventArgs e)
        {
            if (feTargetObject.Visibility == Visibility.Collapsed)
                return;

            Point p = GetDestinationAngleXY(e);
            AnimateToAngleXY(p);
        }

        private Point GetDestinationAngleXY(MouseEventArgs e)
        {
            DetermineSourceWidthAndHeight();
            double x;
            double y;
            var actualPos = e.GetPosition(feSourceObject);
            var pCenter = new Point(sourceWidth / 2, sourceHeight / 2);
            if (ProjectionTowardsMouse == false)
            {
                y = (actualPos.X - pCenter.X) / pCenter.X * AmplitudeAngleX;
                x = (pCenter.Y - actualPos.Y) / pCenter.Y * AmplitudeAngleY;
            }
            else
            {
                y = (pCenter.X - actualPos.X) / pCenter.X * AmplitudeAngleX;
                x = (actualPos.Y - pCenter.Y) / pCenter.Y * AmplitudeAngleY;
            }
            return new Point(x, y);
        }

        private void AnimateToAngleXY(Point xy)
        {
            var slideX = GetDestinationSlide(xy.Y, AmplitudeAngleY, AmplitudeSlideX);
            var slideY = GetDestinationSlide(xy.X, AmplitudeAngleX, AmplitudeSlideY);
            var adjustedSlideX = slideX + basePoint.X;
            var adjustedSlideY = slideY + basePoint.Y;
            playAnimation(feTargetObject, "(Canvas.Left)", tsAnimationTime, adjustedSlideX, Easing);
            playAnimation(feTargetObject, "(Canvas.Top)", tsAnimationTime, adjustedSlideY, Easing);
        }

        public static void playAnimation(FrameworkElement element, string property, TimeSpan time, double value, IEasingFunction easingFunction)
        {
            var sb = new Storyboard();
            sb.Children.Clear();
            var animation = new DoubleAnimation {Duration = time, To = value, EasingFunction = easingFunction};
            Storyboard.SetTargetProperty(animation, new PropertyPath(property));
            Storyboard.SetTarget(animation, element);
            sb.Children.Add(animation);
            sb.Begin();
        }
        private static double GetDestinationSlide(double calculatedAngle, double amplitudeAngle, double amplitudeSlide)
        {
            return calculatedAngle * (amplitudeSlide / amplitudeAngle);
        }
    }
}
