﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Interactivity;

namespace Od.Behaviors.Function
{
    /// <summary>
    /// Provides a trigger using a TimelineMarker with an offset from the end of a MediaElement.
    /// Note that the added TimelineMarker may confuse other uses of TimelineMarkers as triggers.
    /// </summary>
    public class EndCueAction : TriggerBase<MediaElement>
    {
        /// <summary>
        /// Gets or sets the amount of time before the end of the MediaElement to invoke the trigger.
        /// </summary>
        public TimeSpan BeforeEnd
        {
            get;
            set;
        }

        /// <summary>
        /// Override that is called when the trigger is attached.
        /// </summary>
        protected override void OnAttached()
        {
            var mediaElement = AssociatedObject;
            if (mediaElement != null)
            {
                mediaElement.MediaOpened += MediaOpened;
                mediaElement.MarkerReached += MarkerReached;
            }

            base.OnAttached();
        }

        /// <summary>
        /// Override that is called when the trigger is detached.
        /// </summary>
        protected override void OnDetaching()
        {
            var mediaElement = AssociatedObject;
            if (mediaElement != null)
            {
                mediaElement.MediaOpened -= MediaOpened;
                mediaElement.MarkerReached -= MarkerReached;
            }

            base.OnDetaching();
        }

        /// <summary>
        /// The RoutedEvent that is called when the TimelineMarker is reached.
        /// </summary>
        /// <param name="sender">The sender of the event.</param>
        /// <param name="e">The marker that was reached.</param>
        private void MarkerReached(object sender, TimelineMarkerRoutedEventArgs e)
        {
            var mediaElement = sender as MediaElement;
            if (mediaElement != null && e.Marker.Time == (mediaElement.NaturalDuration.TimeSpan - BeforeEnd))
            {
                InvokeActions(null);
            }
        }

        /// <summary>
        /// The RoutedEvent that is called when the Media is first opened.
        /// We have to add the markers here after the media is opened when we can get the Duration from the header information.
        /// </summary>
        /// <param name="sender">The sender of the event.</param>
        /// <param name="e">The routed event.</param>
        private void MediaOpened(object sender, RoutedEventArgs e)
        {
            var mediaElement = sender as MediaElement;
            if (mediaElement == null) return;
            var cueMarker = new TimelineMarker
                                {
                                    Time = mediaElement.NaturalDuration.TimeSpan - BeforeEnd
                                };
            mediaElement.Markers.Add(cueMarker);
        }
    }
}
