﻿using System;
using System.ComponentModel;
using System.IO;
using System.IO.IsolatedStorage;
using System.Reflection;
using System.Runtime.Serialization;
using System.Windows;
using System.Windows.Data;
using System.Windows.Interactivity;
using Od.Behaviors.Utilities.Expression.Samples.Interactivity.DataHelpers;

namespace Od.Behaviors.Function
{
    public class PersistentPropertyBehavior : Behavior<FrameworkElement>
    {
        private BindingListener bindingListener;

        /// <summary>
        /// The property to made persistent
        /// </summary>
        [Description("The property to made persistent")]
        public String PropertyName { get; set; }


        /// <summary>
        /// The name used for the property in the isolated storage
        /// </summary>
        [Description("The name used for the property in the isolated storage")]
        public String PropertyStorageName { get; set; }


        protected override void OnAttached()
        {
            base.OnAttached();

            var persistentProperty =
                AssociatedObject.GetType().GetProperty(PropertyName);

            if (String.IsNullOrEmpty(PropertyName)) PropertyName = "";
            if (String.IsNullOrEmpty(PropertyStorageName))
            {
                throw new Exception("A filename for store property has to be define");
            }

            if (persistentProperty == null)
            {
                throw new Exception("The property defined is invalid");
            }

            if (!persistentProperty.CanWrite || !persistentProperty.CanRead)
            {
                throw new Exception("The property defined cannot be read and write");
            }

            // retrive the dependency property for the property to made persistent
            var persistentDependencyProperty =
                FindDependencyProperty(AssociatedObject, PropertyName);

            if (persistentDependencyProperty == null)
            {
                throw new Exception("There isn't a dependency property defined for the property");
            }

            var storageName = PropertyStorageName;


            var prevValue = retriveValueFromIsolatedStorage(storageName, persistentProperty);

            if (prevValue != null)
                // get the value from isolated storage
                persistentProperty.SetValue(
                    AssociatedObject,
                    prevValue,
                    new object[0]);

            bindingListener = new BindingListener(PropertyChanged)
                                      {
                                          Binding = new Binding(PropertyName)
                                                        {Source = AssociatedObject},
                                          Element = AssociatedObject
                                      };
        }

        void PropertyChanged(object sender, EventArgs e)
        {
            var storageName = PropertyStorageName;

            var persistentProperty =
               AssociatedObject.GetType().GetProperty(PropertyName);
            setValueToIsolatedStorage(storageName, AssociatedObject, persistentProperty);
        }

        protected override void OnDetaching()
        {
            base.OnDetaching();
            if (bindingListener == null) return;
            bindingListener.Binding = null;
            bindingListener.Element = null;
            bindingListener = null;
        }

        static object retriveValueFromIsolatedStorage(string fileName, PropertyInfo property)
        {
            object ret = null;

            using (var isf = IsolatedStorageFile.GetUserStoreForApplication())
            {
                if (isf.FileExists(fileName))
                {
                    using (var isfs = new IsolatedStorageFileStream(fileName, FileMode.Open, isf))
                    {
                        var serializer = new DataContractSerializer(property.PropertyType);
                        ret = serializer.ReadObject(isfs);
                    }
                }
            }
            return ret;

        }

        static void setValueToIsolatedStorage(string fileName, object obj, PropertyInfo property)
        {
            using (var isf = IsolatedStorageFile.GetUserStoreForApplication())
            {
                using (var isfs = new IsolatedStorageFileStream(fileName, FileMode.Create, isf))
                {

                    var serializer = new DataContractSerializer(property.PropertyType);
                    serializer.WriteObject(isfs, property.GetValue(obj, new object[0]));
                }
            }
        }

        static DependencyProperty FindDependencyProperty(object obj, string propertyName)
        {
            if (null != obj && !String.IsNullOrEmpty(propertyName))
            {
                Type typeAttachedTo = obj.GetType();
                FieldInfo fi = null;

                while (null == fi && typeAttachedTo != null)
                {
                    fi = typeAttachedTo.GetField(propertyName + "Property");
                    typeAttachedTo = typeAttachedTo.BaseType;
                }

                if (null != fi)
                {
                    return fi.GetValue(null) as DependencyProperty;
                }
            }
            return null;
        }


    }
}
