﻿namespace Od.Behaviors.Utilities
{
    // (c) Copyright Microsoft Corporation.
    // This source is subject to the Microsoft Permissive License.
    // See http://www.microsoft.com/resources/sharedsource/licensingbasics/sharedsourcelicenses.mspx.
    // All other rights reserved.

    using System;
    using System.Collections.Generic;
    using System.Windows;
    using System.Windows.Data;

    namespace Expression.Samples.Interactivity.DataHelpers
    {

        /// <summary>
        /// Helper class for adding Bindings to non-FrameworkElements
        /// </summary>
        public class BindingListener
        {

            private static readonly List<DependencyPropertyListener> freeListeners = new List<DependencyPropertyListener>();

            /// <summary>
            /// Delegate for when the binding listener has changed.
            /// </summary>
            /// <param name="sender"></param>
            /// <param name="e"></param>
            public delegate void ChangedHandler(object sender, BindingChangedEventArgs e);

            private DependencyPropertyListener listener;
            private readonly ChangedHandler changedHandler;
            private Binding binding;
            private FrameworkElement target;
            private object value;

            /// <summary>
            /// Constructor.
            /// </summary>
            /// <param name="changedHandler">Callback whenever the value of this binding has changed.</param>
            public BindingListener(ChangedHandler changedHandler)
            {
                this.changedHandler = changedHandler;
            }

            /// <summary>
            /// Constructor
            /// </summary>
            public BindingListener()
            {
            }

            /// <summary>
            /// The Binding which is to be evaluated
            /// </summary>
            public Binding Binding
            {
                get { return binding; }
                set
                {
                    binding = value;
                    Attach();
                }
            }

            /// <summary>
            /// The element to be used as the context on which to evaluate the binding.
            /// </summary>
            public FrameworkElement Element
            {
                get { return target; }
                set
                {
                    target = value;
                    Attach();
                }
            }

            /// <summary>
            /// The current value of this binding.
            /// </summary>
            public object Value
            {
                get { return value; }
                set
                {
                    if (listener != null)
                        listener.SetValue(value);
                }
            }


            private void Attach()
            {
                Detach();

                if (target == null || binding == null) return;
                listener = GetListener();
                listener.Attach(target, binding);
            }

            private void Detach()
            {
                if (listener != null)
                    ReturnListener();
            }

            private DependencyPropertyListener GetListener()
            {
                DependencyPropertyListener dlistener;

                if (freeListeners.Count != 0)
                {
                    dlistener = freeListeners[freeListeners.Count - 1];
                    freeListeners.RemoveAt(freeListeners.Count - 1);

                    return dlistener;
                }
                dlistener = new DependencyPropertyListener();

                dlistener.Changed += HandleValueChanged;

                return dlistener;
            }

            private void ReturnListener()
            {
                listener.Changed -= HandleValueChanged;

                freeListeners.Add(listener);

                listener = null;
            }

            private void HandleValueChanged(object sender, BindingChangedEventArgs e)
            {
                value = e.EventArgs.NewValue;

                if (changedHandler != null)
                    changedHandler(this, e);
            }

            private class DependencyPropertyListener
            {

                private readonly DependencyProperty property;
                public event EventHandler<BindingChangedEventArgs> Changed;

                private static int index;
                private FrameworkElement target;

                public DependencyPropertyListener()
                {
                    property = DependencyProperty.RegisterAttached("DependencyPropertyListener" + index++, typeof(object), 
                        typeof(DependencyPropertyListener), new PropertyMetadata(null, HandleValueChanged));
                }

                private void HandleValueChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
                {

                    if (Changed != null)
                        Changed(this, new BindingChangedEventArgs(e));
                }

                public void Attach(FrameworkElement element, Binding binding)
                {
                    if (target != null)
                        throw new Exception("Cannot attach an already attached listener");

                    target = element;

                    target.SetBinding(property, binding);
                }

                public void Detach()
                {
                    target.ClearValue(property);
                    target = null;
                }

                public void SetValue(object value)
                {
                    target.SetValue(property, value);
                }
            }
        }

        /// <summary>
        /// Event args for when binding values change.
        /// </summary>
        public class BindingChangedEventArgs : EventArgs
        {
            /// <summary>
            /// Constructor
            /// </summary>
            /// <param name="e"></param>
            public BindingChangedEventArgs(DependencyPropertyChangedEventArgs e)
            {
                EventArgs = e;
            }

            /// <summary>
            /// Original event args.
            /// </summary>
            public DependencyPropertyChangedEventArgs EventArgs { get; private set; }
        }
    }

}
