﻿using System;
using System.Windows.Controls;
using System.Windows.Input;

namespace Od.Behaviors.Utilities
{
    /// <summary>
    /// The DelegatingCommand provides an implementation of <see cref="ICommand"/>
    /// that can delegate <see cref="ICommand.CanExecute">CanExecute</see> and
    /// <see cref="ICommand.Execute">Execute</see> calls to listeners. As 
    /// opposed to the RoutedCommand, it does not require any custom wiring to 
    /// the UI, just bind an instance to any <see cref="ICommand"/> (e.g.
    /// <see cref="Button"/>) using XAML. This is  useful when implementing the
    /// MVVM pattern within WPF or Silverlight. The ViewModel can
    /// expose a DelegatingCommand as a property and the View can bind to it.
    /// Anyone aware of the instance can handle the CanExecuteRequested and/or
    /// ExecuteRequested events.    
    /// </summary>
    public class DelegatingCommand:ICommand
    {
        /// <summary>
        /// Delegate to be called in response to <see cref="Execute"/> being 
        /// invoked.
        /// </summary>
        public Action<object> ExecuteRequested { get; set; }

        /// <summary>
        /// Delegate to be called in response to <see cref="CanExecute"/> being
        /// invoked.
        /// </summary>
        public Func<object,bool>CanExecuteRequested { get; set; }

        /// <summary>
        /// Defines the method that determines whether the command can execute 
        /// in its current state.
        /// </summary>
        /// <returns>
        /// true if this command can be executed; otherwise, false.
        /// </returns>
        /// <param name="parameter">
        /// Data used by the command. If the command does not require data to
        /// be passed, this object can be set to null. 
        /// </param>
        public bool CanExecute(object parameter)
        {
            return CanExecuteRequested == null || CanExecuteRequested(parameter);
        }

        /// <summary>
        /// Defines the method to be called when the command is invoked.
        /// </summary>
        /// <param name="parameter">
        /// Data used by the command. If the command does not require data to 
        /// be passed, this object can be set to null. 
        /// </param>
        public void Execute(object parameter)
        {
            if (ExecuteRequested != null)
                ExecuteRequested(parameter);
        }

        /// <summary>
        /// Raises the can execute changed event.
        /// When the application state changes the "can execute" status of
        /// the command, it must call RaiseCanExecuteChanged so UI element
        /// bound to the command can update its own state.
        /// (added OD jun 2010)
        /// </summary>
        public void RaiseCanExecuteChanged()
        {
            var c = CanExecuteChanged;
            if (c == null) return;
            c(this,new EventArgs());
        }

        public event EventHandler CanExecuteChanged;
    }
}
